# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx

from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.jsonable_encoder import jsonable_encoder
from .core.remove_none_from_dict import remove_none_from_dict
from .environment import JulepApiEnvironment
from .types.agent import Agent
from .types.agent_default_settings import AgentDefaultSettings
from .types.chat_input_data_tool_choice import ChatInputDataToolChoice
from .types.chat_response import ChatResponse
from .types.chat_settings_preset import ChatSettingsPreset
from .types.chat_settings_response_format import ChatSettingsResponseFormat
from .types.chat_settings_stop import ChatSettingsStop
from .types.create_agent_request_instructions import CreateAgentRequestInstructions
from .types.create_agent_request_metadata import CreateAgentRequestMetadata
from .types.create_doc import CreateDoc
from .types.create_session_request_metadata import CreateSessionRequestMetadata
from .types.create_tool_request import CreateToolRequest
from .types.create_user_request_metadata import CreateUserRequestMetadata
from .types.function_def import FunctionDef
from .types.get_agent_docs_request_order import GetAgentDocsRequestOrder
from .types.get_agent_docs_request_sort_by import GetAgentDocsRequestSortBy
from .types.get_agent_docs_response import GetAgentDocsResponse
from .types.get_agent_memories_response import GetAgentMemoriesResponse
from .types.get_agent_tools_response import GetAgentToolsResponse
from .types.get_history_response import GetHistoryResponse
from .types.get_suggestions_response import GetSuggestionsResponse
from .types.get_user_docs_request_order import GetUserDocsRequestOrder
from .types.get_user_docs_request_sort_by import GetUserDocsRequestSortBy
from .types.get_user_docs_response import GetUserDocsResponse
from .types.input_chat_ml_message import InputChatMlMessage
from .types.job_status import JobStatus
from .types.list_agents_request_order import ListAgentsRequestOrder
from .types.list_agents_request_sort_by import ListAgentsRequestSortBy
from .types.list_agents_response import ListAgentsResponse
from .types.list_sessions_request_order import ListSessionsRequestOrder
from .types.list_sessions_request_sort_by import ListSessionsRequestSortBy
from .types.list_sessions_response import ListSessionsResponse
from .types.list_users_request_order import ListUsersRequestOrder
from .types.list_users_request_sort_by import ListUsersRequestSortBy
from .types.list_users_response import ListUsersResponse
from .types.partial_function_def import PartialFunctionDef
from .types.patch_agent_request_instructions import PatchAgentRequestInstructions
from .types.patch_agent_request_metadata import PatchAgentRequestMetadata
from .types.patch_session_request_metadata import PatchSessionRequestMetadata
from .types.patch_user_request_metadata import PatchUserRequestMetadata
from .types.resource_created_response import ResourceCreatedResponse
from .types.resource_updated_response import ResourceUpdatedResponse
from .types.session import Session
from .types.tool import Tool
from .types.update_agent_request_instructions import UpdateAgentRequestInstructions
from .types.update_agent_request_metadata import UpdateAgentRequestMetadata
from .types.update_session_request_metadata import UpdateSessionRequestMetadata
from .types.update_user_request_metadata import UpdateUserRequestMetadata
from .types.user import User

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class JulepApi:
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: JulepApiEnvironment = JulepApiEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = 300,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=(
                httpx.Client(timeout=timeout) if httpx_client is None else httpx_client
            ),
        )

    def list_sessions(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListSessionsRequestSortBy] = None,
        order: typing.Optional[ListSessionsRequestOrder] = None,
    ) -> ListSessionsResponse:
        """
        List sessions created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of sessions to return

            - offset: typing.Optional[int]. Number of sessions to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListSessionsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListSessionsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.list_sessions()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sessions"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListSessionsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_session(
        self,
        *,
        user_id: typing.Optional[str] = OMIT,
        agent_id: str,
        situation: typing.Optional[str] = OMIT,
        metadata: typing.Optional[CreateSessionRequestMetadata] = OMIT,
        render_templates: typing.Optional[bool] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a session between an agent and a user

        Parameters:
            - user_id: typing.Optional[str]. (Optional) User ID of user to associate with this session

            - agent_id: str. Agent ID of agent to associate with this session

            - situation: typing.Optional[str]. A specific situation that sets the background for this session

            - metadata: typing.Optional[CreateSessionRequestMetadata]. Optional metadata

            - render_templates: typing.Optional[bool]. Render system and assistant message content as jinja templates
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_session(
            agent_id="agent_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"agent_id": agent_id}
        if user_id is not OMIT:
            _request["user_id"] = user_id
        if situation is not OMIT:
            _request["situation"] = situation
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if render_templates is not OMIT:
            _request["render_templates"] = render_templates
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sessions"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_users(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListUsersRequestSortBy] = None,
        order: typing.Optional[ListUsersRequestOrder] = None,
    ) -> ListUsersResponse:
        """
        List users created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of items to return

            - offset: typing.Optional[int]. Number of items to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListUsersRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListUsersRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.list_users()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListUsersResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_user(
        self,
        *,
        name: typing.Optional[str] = OMIT,
        about: typing.Optional[str] = OMIT,
        docs: typing.Optional[typing.List[CreateDoc]] = OMIT,
        metadata: typing.Optional[CreateUserRequestMetadata] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a new user

        Parameters:
            - name: typing.Optional[str]. Name of the user

            - about: typing.Optional[str]. About the user

            - docs: typing.Optional[typing.List[CreateDoc]]. List of docs about user

            - metadata: typing.Optional[CreateUserRequestMetadata]. (Optional) metadata
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_user()
        """
        _request: typing.Dict[str, typing.Any] = {}
        if name is not OMIT:
            _request["name"] = name
        if about is not OMIT:
            _request["about"] = about
        if docs is not OMIT:
            _request["docs"] = docs
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_agents(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListAgentsRequestSortBy] = None,
        order: typing.Optional[ListAgentsRequestOrder] = None,
    ) -> ListAgentsResponse:
        """
        List agents created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of items to return

            - offset: typing.Optional[int]. Number of items to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListAgentsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListAgentsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.list_agents()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "agents"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListAgentsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_agent(
        self,
        *,
        name: str,
        about: typing.Optional[str] = OMIT,
        tools: typing.Optional[typing.List[CreateToolRequest]] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        model: typing.Optional[str] = OMIT,
        docs: typing.Optional[typing.List[CreateDoc]] = OMIT,
        metadata: typing.Optional[CreateAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[CreateAgentRequestInstructions] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a new agent

        Parameters:
            - name: str. Name of the agent

            - about: typing.Optional[str]. About the agent

            - tools: typing.Optional[typing.List[CreateToolRequest]]. A list of tools the model may call. Currently, only `function`s are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for.

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - docs: typing.Optional[typing.List[CreateDoc]]. List of docs about agent

            - metadata: typing.Optional[CreateAgentRequestMetadata]. (Optional) metadata

            - instructions: typing.Optional[CreateAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_agent(
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name}
        if about is not OMIT:
            _request["about"] = about
        if tools is not OMIT:
            _request["tools"] = tools
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if model is not OMIT:
            _request["model"] = model
        if docs is not OMIT:
            _request["docs"] = docs
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "agents"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_session(self, session_id: str) -> Session:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_session(
            session_id="session_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Session, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_session(
        self,
        session_id: str,
        *,
        situation: str,
        metadata: typing.Optional[UpdateSessionRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - session_id: str.

            - situation: str. Updated situation for this session

            - metadata: typing.Optional[UpdateSessionRequestMetadata]. Optional metadata
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.update_session(
            session_id="session_id",
            situation="situation",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"situation": situation}
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_session(self, session_id: str) -> None:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_session(
            session_id="session_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_session(
        self,
        session_id: str,
        *,
        situation: typing.Optional[str] = OMIT,
        metadata: typing.Optional[PatchSessionRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - session_id: str.

            - situation: typing.Optional[str]. Updated situation for this session

            - metadata: typing.Optional[PatchSessionRequestMetadata]. Optional metadata
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.patch_session(
            session_id="session_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if situation is not OMIT:
            _request["situation"] = situation
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_suggestions(
        self,
        session_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetSuggestionsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - session_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_suggestions(
            session_id="session_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/suggestions",
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetSuggestionsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_history(
        self,
        session_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetHistoryResponse:
        """
        Sorted (created_at ascending)

        Parameters:
            - session_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_history(
            session_id="session_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/history",
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetHistoryResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_session_history(self, session_id: str) -> None:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_session_history(
            session_id="session_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/history",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def chat(
        self,
        session_id: str,
        *,
        messages: typing.List[InputChatMlMessage],
        tools: typing.Optional[typing.List[Tool]] = OMIT,
        tool_choice: typing.Optional[ChatInputDataToolChoice] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        length_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[int]]] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatSettingsResponseFormat] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[ChatSettingsStop] = OMIT,
        stream: typing.Optional[bool] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        min_p: typing.Optional[float] = OMIT,
        preset: typing.Optional[ChatSettingsPreset] = OMIT,
        recall: typing.Optional[bool] = OMIT,
        record: typing.Optional[bool] = OMIT,
        remember: typing.Optional[bool] = OMIT,
    ) -> ChatResponse:
        """


        Parameters:
            - session_id: str.

            - messages: typing.List[InputChatMlMessage]. A list of new input messages comprising the conversation so far.

            - tools: typing.Optional[typing.List[Tool]]. (Advanced) List of tools that are provided in addition to agent's default set of tools. Functions of same name in agent set are overriden

            - tool_choice: typing.Optional[ChatInputDataToolChoice]. Can be one of existing tools given to the agent earlier or the ones included in the request

            - frequency_penalty: typing.Optional[float]. (OpenAI-like) Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - length_penalty: typing.Optional[float]. (Huggingface-like) Number between 0 and 2.0. 1.0 is neutral and values larger than that penalize number of tokens generated.

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[int]]]. Modify the likelihood of specified tokens appearing in the completion.

                                                                                   Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token.
            - max_tokens: typing.Optional[int]. The maximum number of tokens to generate in the chat completion.

                                                The total length of input tokens and generated tokens is limited by the model's context length.
            - presence_penalty: typing.Optional[float]. (OpenAI-like) Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - repetition_penalty: typing.Optional[float]. (Huggingface-like) Number between 0 and 2.0. 1.0 is neutral and values larger than that penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - response_format: typing.Optional[ChatSettingsResponseFormat]. An object specifying the format that the model must output.

                                                                            Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.

                                                                            **Important:** when using JSON mode, you **must** also instruct the model to produce JSON yourself via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request. Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length.
            - seed: typing.Optional[int]. This feature is in Beta.
                                          If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result.
                                          Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.
            - stop: typing.Optional[ChatSettingsStop]. Up to 4 sequences where the API will stop generating further tokens.

            - stream: typing.Optional[bool]. If set, partial message deltas will be sent, like in ChatGPT. Tokens will be sent as data-only [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format) as they become available, with the stream terminated by a `data: [DONE]` message. [Example Python code](https://cookbook.openai.com/examples/how_to_stream_completions).

            - temperature: typing.Optional[float]. What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.

            - top_p: typing.Optional[float]. Defaults to 1 An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or temperature but not both.

            - min_p: typing.Optional[float]. Minimum probability compared to leading token to be considered

            - preset: typing.Optional[ChatSettingsPreset]. Generation preset name (problem_solving|conversational|fun|prose|creative|business|deterministic|code|multilingual)

            - recall: typing.Optional[bool]. Whether previous memories should be recalled or not

            - record: typing.Optional[bool]. Whether this interaction should be recorded in history or not

            - remember: typing.Optional[bool]. Whether this interaction should form memories or not
        ---
        from julep import InputChatMlMessage, InputChatMlMessageRole
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.chat(
            session_id="session_id",
            min_p=0.01,
            messages=[
                InputChatMlMessage(
                    role=InputChatMlMessageRole.USER,
                    content="content",
                )
            ],
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages}
        if tools is not OMIT:
            _request["tools"] = tools
        if tool_choice is not OMIT:
            _request["tool_choice"] = tool_choice
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if length_penalty is not OMIT:
            _request["length_penalty"] = length_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if stream is not OMIT:
            _request["stream"] = stream
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if min_p is not OMIT:
            _request["min_p"] = min_p
        if preset is not OMIT:
            _request["preset"] = preset.value  # type: ignore
        if recall is not OMIT:
            _request["recall"] = recall
        if record is not OMIT:
            _request["record"] = record
        if remember is not OMIT:
            _request["remember"] = remember
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}/chat"
            ),
            json=jsonable_encoder(_request),
            headers=remove_none_from_dict(
                {**self._client_wrapper.get_headers(), "Accept": "application/json"}
            ),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ChatResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_agent_memories(
        self,
        agent_id: str,
        *,
        query: str,
        user_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetAgentMemoriesResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - query: str.

            - user_id: typing.Optional[str].

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_agent_memories(
            agent_id="agent_id",
            query="query",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/memories"
            ),
            params=remove_none_from_dict(
                {"query": query, "user_id": user_id, "limit": limit, "offset": offset}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentMemoriesResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_user(self, user_id: str) -> User:
        """


        Parameters:
            - user_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_user(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(User, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_user(
        self,
        user_id: str,
        *,
        about: str,
        name: str,
        metadata: typing.Optional[UpdateUserRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - user_id: str.

            - about: str. About the user

            - name: str. Name of the user

            - metadata: typing.Optional[UpdateUserRequestMetadata]. Optional metadata
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.update_user(
            user_id="user_id",
            about="about",
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"about": about, "name": name}
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_user(self, user_id: str) -> None:
        """


        Parameters:
            - user_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_user(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_user(
        self,
        user_id: str,
        *,
        about: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        metadata: typing.Optional[PatchUserRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - user_id: str.

            - about: typing.Optional[str]. About the user

            - name: typing.Optional[str]. Name of the user

            - metadata: typing.Optional[PatchUserRequestMetadata]. Optional metadata
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.patch_user(
            user_id="user_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if about is not OMIT:
            _request["about"] = about
        if name is not OMIT:
            _request["name"] = name
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_agent(self, agent_id: str) -> Agent:
        """


        Parameters:
            - agent_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_agent(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Agent, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_agent(
        self,
        agent_id: str,
        *,
        about: str,
        name: str,
        model: typing.Optional[str] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        metadata: typing.Optional[UpdateAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[UpdateAgentRequestInstructions] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - about: str. About the agent

            - name: str. Name of the agent

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - metadata: typing.Optional[UpdateAgentRequestMetadata]. Optional metadata

            - instructions: typing.Optional[UpdateAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.update_agent(
            agent_id="agent_id",
            about="about",
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"about": about, "name": name}
        if model is not OMIT:
            _request["model"] = model
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_agent(self, agent_id: str) -> None:
        """


        Parameters:
            - agent_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_agent(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_agent(
        self,
        agent_id: str,
        *,
        about: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        model: typing.Optional[str] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        metadata: typing.Optional[PatchAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[PatchAgentRequestInstructions] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - about: typing.Optional[str]. About the agent

            - name: typing.Optional[str]. Name of the agent

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - metadata: typing.Optional[PatchAgentRequestMetadata]. Optional metadata

            - instructions: typing.Optional[PatchAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.patch_agent(
            agent_id="agent_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if about is not OMIT:
            _request["about"] = about
        if name is not OMIT:
            _request["name"] = name
        if model is not OMIT:
            _request["model"] = model
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_agent_docs(
        self,
        agent_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[GetAgentDocsRequestSortBy] = None,
        order: typing.Optional[GetAgentDocsRequestOrder] = None,
    ) -> GetAgentDocsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[GetAgentDocsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[GetAgentDocsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_agent_docs(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/docs"
            ),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentDocsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_agent_doc(
        self, agent_id: str, *, request: CreateDoc
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - agent_id: str.

            - request: CreateDoc.
        ---
        from julep import CreateDoc
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_agent_doc(
            agent_id="agent_id",
            request=CreateDoc(
                title="title",
                content="content",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/docs"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_user_docs(
        self,
        user_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[GetUserDocsRequestSortBy] = None,
        order: typing.Optional[GetUserDocsRequestOrder] = None,
    ) -> GetUserDocsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - user_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[GetUserDocsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[GetUserDocsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_user_docs(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/docs"
            ),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetUserDocsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_user_doc(
        self, user_id: str, *, request: CreateDoc
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - user_id: str.

            - request: CreateDoc.
        ---
        from julep import CreateDoc
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_user_doc(
            user_id="user_id",
            request=CreateDoc(
                title="title",
                content="content",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/docs"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_user_doc(self, user_id: str, doc_id: str) -> None:
        """


        Parameters:
            - user_id: str.

            - doc_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_user_doc(
            user_id="user_id",
            doc_id="doc_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"users/{user_id}/docs/{doc_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_agent_doc(self, agent_id: str, doc_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - doc_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_agent_doc(
            agent_id="agent_id",
            doc_id="doc_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/docs/{doc_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_agent_memory(self, agent_id: str, memory_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - memory_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_agent_memory(
            agent_id="agent_id",
            memory_id="memory_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/memories/{memory_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_agent_tools(
        self,
        agent_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetAgentToolsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_agent_tools(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/tools"
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentToolsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_agent_tool(
        self, agent_id: str, *, request: CreateToolRequest
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - agent_id: str.

            - request: CreateToolRequest.
        ---
        from julep import CreateToolRequest, CreateToolRequestType, FunctionDef
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.create_agent_tool(
            agent_id="agent_id",
            request=CreateToolRequest(
                type=CreateToolRequestType.FUNCTION,
                function=FunctionDef(
                    name="name",
                    parameters={},
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/tools"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_agent_tool(
        self, agent_id: str, tool_id: str, *, function: FunctionDef
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.

            - function: FunctionDef. Function definition and parameters
        ---
        from julep import FunctionDef
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.update_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
            function=FunctionDef(
                name="name",
                parameters={},
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            json=jsonable_encoder({"function": function}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_agent_tool(self, agent_id: str, tool_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.delete_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch_agent_tool(
        self, agent_id: str, tool_id: str, *, function: PartialFunctionDef
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.

            - function: PartialFunctionDef. Function definition and parameters
        ---
        from julep import PartialFunctionDef
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.patch_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
            function=PartialFunctionDef(),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            json=jsonable_encoder({"function": function}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_job_status(self, job_id: str) -> JobStatus:
        """


        Parameters:
            - job_id: str.
        ---
        from julep.client import JulepApi

        client = JulepApi(
            api_key="YOUR_API_KEY",
        )
        client.get_job_status(
            job_id="job_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"jobs/{job_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(JobStatus, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncJulepApi:
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: JulepApiEnvironment = JulepApiEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = 300,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=(
                httpx.AsyncClient(timeout=timeout)
                if httpx_client is None
                else httpx_client
            ),
        )

    async def list_sessions(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListSessionsRequestSortBy] = None,
        order: typing.Optional[ListSessionsRequestOrder] = None,
    ) -> ListSessionsResponse:
        """
        List sessions created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of sessions to return

            - offset: typing.Optional[int]. Number of sessions to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListSessionsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListSessionsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.list_sessions()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sessions"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListSessionsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_session(
        self,
        *,
        user_id: typing.Optional[str] = OMIT,
        agent_id: str,
        situation: typing.Optional[str] = OMIT,
        metadata: typing.Optional[CreateSessionRequestMetadata] = OMIT,
        render_templates: typing.Optional[bool] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a session between an agent and a user

        Parameters:
            - user_id: typing.Optional[str]. (Optional) User ID of user to associate with this session

            - agent_id: str. Agent ID of agent to associate with this session

            - situation: typing.Optional[str]. A specific situation that sets the background for this session

            - metadata: typing.Optional[CreateSessionRequestMetadata]. Optional metadata

            - render_templates: typing.Optional[bool]. Render system and assistant message content as jinja templates
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_session(
            agent_id="agent_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"agent_id": agent_id}
        if user_id is not OMIT:
            _request["user_id"] = user_id
        if situation is not OMIT:
            _request["situation"] = situation
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if render_templates is not OMIT:
            _request["render_templates"] = render_templates
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "sessions"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_users(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListUsersRequestSortBy] = None,
        order: typing.Optional[ListUsersRequestOrder] = None,
    ) -> ListUsersResponse:
        """
        List users created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of items to return

            - offset: typing.Optional[int]. Number of items to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListUsersRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListUsersRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.list_users()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListUsersResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_user(
        self,
        *,
        name: typing.Optional[str] = OMIT,
        about: typing.Optional[str] = OMIT,
        docs: typing.Optional[typing.List[CreateDoc]] = OMIT,
        metadata: typing.Optional[CreateUserRequestMetadata] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a new user

        Parameters:
            - name: typing.Optional[str]. Name of the user

            - about: typing.Optional[str]. About the user

            - docs: typing.Optional[typing.List[CreateDoc]]. List of docs about user

            - metadata: typing.Optional[CreateUserRequestMetadata]. (Optional) metadata
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_user()
        """
        _request: typing.Dict[str, typing.Any] = {}
        if name is not OMIT:
            _request["name"] = name
        if about is not OMIT:
            _request["about"] = about
        if docs is not OMIT:
            _request["docs"] = docs
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_agents(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ListAgentsRequestSortBy] = None,
        order: typing.Optional[ListAgentsRequestOrder] = None,
    ) -> ListAgentsResponse:
        """
        List agents created (use limit/offset pagination to get large number of sessions; sorted by descending order of `created_at` by default)

        Parameters:
            - limit: typing.Optional[int]. Number of items to return

            - offset: typing.Optional[int]. Number of items to skip (sorted created_at descending order)

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[ListAgentsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[ListAgentsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.list_agents()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "agents"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListAgentsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_agent(
        self,
        *,
        name: str,
        about: typing.Optional[str] = OMIT,
        tools: typing.Optional[typing.List[CreateToolRequest]] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        model: typing.Optional[str] = OMIT,
        docs: typing.Optional[typing.List[CreateDoc]] = OMIT,
        metadata: typing.Optional[CreateAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[CreateAgentRequestInstructions] = OMIT,
    ) -> ResourceCreatedResponse:
        """
        Create a new agent

        Parameters:
            - name: str. Name of the agent

            - about: typing.Optional[str]. About the agent

            - tools: typing.Optional[typing.List[CreateToolRequest]]. A list of tools the model may call. Currently, only `function`s are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for.

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - docs: typing.Optional[typing.List[CreateDoc]]. List of docs about agent

            - metadata: typing.Optional[CreateAgentRequestMetadata]. (Optional) metadata

            - instructions: typing.Optional[CreateAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_agent(
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"name": name}
        if about is not OMIT:
            _request["about"] = about
        if tools is not OMIT:
            _request["tools"] = tools
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if model is not OMIT:
            _request["model"] = model
        if docs is not OMIT:
            _request["docs"] = docs
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "agents"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_session(self, session_id: str) -> Session:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_session(
            session_id="session_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Session, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_session(
        self,
        session_id: str,
        *,
        situation: str,
        metadata: typing.Optional[UpdateSessionRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - session_id: str.

            - situation: str. Updated situation for this session

            - metadata: typing.Optional[UpdateSessionRequestMetadata]. Optional metadata
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.update_session(
            session_id="session_id",
            situation="situation",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"situation": situation}
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_session(self, session_id: str) -> None:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_session(
            session_id="session_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_session(
        self,
        session_id: str,
        *,
        situation: typing.Optional[str] = OMIT,
        metadata: typing.Optional[PatchSessionRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - session_id: str.

            - situation: typing.Optional[str]. Updated situation for this session

            - metadata: typing.Optional[PatchSessionRequestMetadata]. Optional metadata
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.patch_session(
            session_id="session_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if situation is not OMIT:
            _request["situation"] = situation
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_suggestions(
        self,
        session_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetSuggestionsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - session_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_suggestions(
            session_id="session_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/suggestions",
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetSuggestionsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_history(
        self,
        session_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetHistoryResponse:
        """
        Sorted (created_at ascending)

        Parameters:
            - session_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_history(
            session_id="session_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/history",
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetHistoryResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_session_history(self, session_id: str) -> None:
        """


        Parameters:
            - session_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_session_history(
            session_id="session_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"sessions/{session_id}/history",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def chat(
        self,
        session_id: str,
        *,
        messages: typing.List[InputChatMlMessage],
        tools: typing.Optional[typing.List[Tool]] = OMIT,
        tool_choice: typing.Optional[ChatInputDataToolChoice] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        length_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[int]]] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatSettingsResponseFormat] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[ChatSettingsStop] = OMIT,
        stream: typing.Optional[bool] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        min_p: typing.Optional[float] = OMIT,
        preset: typing.Optional[ChatSettingsPreset] = OMIT,
        recall: typing.Optional[bool] = OMIT,
        record: typing.Optional[bool] = OMIT,
        remember: typing.Optional[bool] = OMIT,
    ) -> ChatResponse:
        """


        Parameters:
            - session_id: str.

            - messages: typing.List[InputChatMlMessage]. A list of new input messages comprising the conversation so far.

            - tools: typing.Optional[typing.List[Tool]]. (Advanced) List of tools that are provided in addition to agent's default set of tools. Functions of same name in agent set are overriden

            - tool_choice: typing.Optional[ChatInputDataToolChoice]. Can be one of existing tools given to the agent earlier or the ones included in the request

            - frequency_penalty: typing.Optional[float]. (OpenAI-like) Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - length_penalty: typing.Optional[float]. (Huggingface-like) Number between 0 and 2.0. 1.0 is neutral and values larger than that penalize number of tokens generated.

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[int]]]. Modify the likelihood of specified tokens appearing in the completion.

                                                                                   Accepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token.
            - max_tokens: typing.Optional[int]. The maximum number of tokens to generate in the chat completion.

                                                The total length of input tokens and generated tokens is limited by the model's context length.
            - presence_penalty: typing.Optional[float]. (OpenAI-like) Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - repetition_penalty: typing.Optional[float]. (Huggingface-like) Number between 0 and 2.0. 1.0 is neutral and values larger than that penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.

            - response_format: typing.Optional[ChatSettingsResponseFormat]. An object specifying the format that the model must output.

                                                                            Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.

                                                                            **Important:** when using JSON mode, you **must** also instruct the model to produce JSON yourself via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request. Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length.
            - seed: typing.Optional[int]. This feature is in Beta.
                                          If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result.
                                          Determinism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.
            - stop: typing.Optional[ChatSettingsStop]. Up to 4 sequences where the API will stop generating further tokens.

            - stream: typing.Optional[bool]. If set, partial message deltas will be sent, like in ChatGPT. Tokens will be sent as data-only [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format) as they become available, with the stream terminated by a `data: [DONE]` message. [Example Python code](https://cookbook.openai.com/examples/how_to_stream_completions).

            - temperature: typing.Optional[float]. What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.

            - top_p: typing.Optional[float]. Defaults to 1 An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered. We generally recommend altering this or temperature but not both.

            - min_p: typing.Optional[float]. Minimum probability compared to leading token to be considered

            - preset: typing.Optional[ChatSettingsPreset]. Generation preset name (problem_solving|conversational|fun|prose|creative|business|deterministic|code|multilingual)

            - recall: typing.Optional[bool]. Whether previous memories should be recalled or not

            - record: typing.Optional[bool]. Whether this interaction should be recorded in history or not

            - remember: typing.Optional[bool]. Whether this interaction should form memories or not
        ---
        from julep import InputChatMlMessage, InputChatMlMessageRole
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.chat(
            session_id="session_id",
            min_p=0.01,
            messages=[
                InputChatMlMessage(
                    role=InputChatMlMessageRole.USER,
                    content="content",
                )
            ],
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages}
        if tools is not OMIT:
            _request["tools"] = tools
        if tool_choice is not OMIT:
            _request["tool_choice"] = tool_choice
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if length_penalty is not OMIT:
            _request["length_penalty"] = length_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if stream is not OMIT:
            _request["stream"] = stream
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if min_p is not OMIT:
            _request["min_p"] = min_p
        if preset is not OMIT:
            _request["preset"] = preset.value  # type: ignore
        if recall is not OMIT:
            _request["recall"] = recall
        if record is not OMIT:
            _request["record"] = record
        if remember is not OMIT:
            _request["remember"] = remember
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sessions/{session_id}/chat"
            ),
            json=jsonable_encoder(_request),
            headers=remove_none_from_dict(
                {**self._client_wrapper.get_headers(), "Accept": "application/json"}
            ),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ChatResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_agent_memories(
        self,
        agent_id: str,
        *,
        query: str,
        user_id: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetAgentMemoriesResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - query: str.

            - user_id: typing.Optional[str].

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_agent_memories(
            agent_id="agent_id",
            query="query",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/memories"
            ),
            params=remove_none_from_dict(
                {"query": query, "user_id": user_id, "limit": limit, "offset": offset}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentMemoriesResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_user(self, user_id: str) -> User:
        """


        Parameters:
            - user_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_user(
            user_id="user_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(User, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_user(
        self,
        user_id: str,
        *,
        about: str,
        name: str,
        metadata: typing.Optional[UpdateUserRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - user_id: str.

            - about: str. About the user

            - name: str. Name of the user

            - metadata: typing.Optional[UpdateUserRequestMetadata]. Optional metadata
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.update_user(
            user_id="user_id",
            about="about",
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"about": about, "name": name}
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_user(self, user_id: str) -> None:
        """


        Parameters:
            - user_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_user(
            user_id="user_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_user(
        self,
        user_id: str,
        *,
        about: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        metadata: typing.Optional[PatchUserRequestMetadata] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - user_id: str.

            - about: typing.Optional[str]. About the user

            - name: typing.Optional[str]. Name of the user

            - metadata: typing.Optional[PatchUserRequestMetadata]. Optional metadata
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.patch_user(
            user_id="user_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if about is not OMIT:
            _request["about"] = about
        if name is not OMIT:
            _request["name"] = name
        if metadata is not OMIT:
            _request["metadata"] = metadata
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_agent(self, agent_id: str) -> Agent:
        """


        Parameters:
            - agent_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_agent(
            agent_id="agent_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Agent, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_agent(
        self,
        agent_id: str,
        *,
        about: str,
        name: str,
        model: typing.Optional[str] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        metadata: typing.Optional[UpdateAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[UpdateAgentRequestInstructions] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - about: str. About the agent

            - name: str. Name of the agent

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - metadata: typing.Optional[UpdateAgentRequestMetadata]. Optional metadata

            - instructions: typing.Optional[UpdateAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.update_agent(
            agent_id="agent_id",
            about="about",
            name="name",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"about": about, "name": name}
        if model is not OMIT:
            _request["model"] = model
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_agent(self, agent_id: str) -> None:
        """


        Parameters:
            - agent_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_agent(
            agent_id="agent_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_agent(
        self,
        agent_id: str,
        *,
        about: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        model: typing.Optional[str] = OMIT,
        default_settings: typing.Optional[AgentDefaultSettings] = OMIT,
        metadata: typing.Optional[PatchAgentRequestMetadata] = OMIT,
        instructions: typing.Optional[PatchAgentRequestInstructions] = OMIT,
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - about: typing.Optional[str]. About the agent

            - name: typing.Optional[str]. Name of the agent

            - model: typing.Optional[str]. Name of the model that the agent is supposed to use

            - default_settings: typing.Optional[AgentDefaultSettings]. Default model settings to start every session with

            - metadata: typing.Optional[PatchAgentRequestMetadata]. Optional metadata

            - instructions: typing.Optional[PatchAgentRequestInstructions]. Instructions for the agent
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.patch_agent(
            agent_id="agent_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if about is not OMIT:
            _request["about"] = about
        if name is not OMIT:
            _request["name"] = name
        if model is not OMIT:
            _request["model"] = model
        if default_settings is not OMIT:
            _request["default_settings"] = default_settings
        if metadata is not OMIT:
            _request["metadata"] = metadata
        if instructions is not OMIT:
            _request["instructions"] = instructions
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_agent_docs(
        self,
        agent_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[GetAgentDocsRequestSortBy] = None,
        order: typing.Optional[GetAgentDocsRequestOrder] = None,
    ) -> GetAgentDocsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[GetAgentDocsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[GetAgentDocsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_agent_docs(
            agent_id="agent_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/docs"
            ),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentDocsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_agent_doc(
        self, agent_id: str, *, request: CreateDoc
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - agent_id: str.

            - request: CreateDoc.
        ---
        from julep import CreateDoc
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_agent_doc(
            agent_id="agent_id",
            request=CreateDoc(
                title="title",
                content="content",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/docs"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_user_docs(
        self,
        user_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        metadata_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[GetUserDocsRequestSortBy] = None,
        order: typing.Optional[GetUserDocsRequestOrder] = None,
    ) -> GetUserDocsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - user_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - metadata_filter: typing.Optional[str]. JSON object that should be used to filter objects by metadata

            - sort_by: typing.Optional[GetUserDocsRequestSortBy]. Which field to sort by: `created_at` or `updated_at`

            - order: typing.Optional[GetUserDocsRequestOrder]. Which order should the sort be: `asc` (ascending) or `desc` (descending)
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_user_docs(
            user_id="user_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/docs"
            ),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "offset": offset,
                    "metadata_filter": metadata_filter,
                    "sort_by": sort_by,
                    "order": order,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetUserDocsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_user_doc(
        self, user_id: str, *, request: CreateDoc
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - user_id: str.

            - request: CreateDoc.
        ---
        from julep import CreateDoc
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_user_doc(
            user_id="user_id",
            request=CreateDoc(
                title="title",
                content="content",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/docs"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_user_doc(self, user_id: str, doc_id: str) -> None:
        """


        Parameters:
            - user_id: str.

            - doc_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_user_doc(
            user_id="user_id",
            doc_id="doc_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"users/{user_id}/docs/{doc_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_agent_doc(self, agent_id: str, doc_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - doc_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_agent_doc(
            agent_id="agent_id",
            doc_id="doc_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/docs/{doc_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_agent_memory(self, agent_id: str, memory_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - memory_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_agent_memory(
            agent_id="agent_id",
            memory_id="memory_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/memories/{memory_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_agent_tools(
        self,
        agent_id: str,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
    ) -> GetAgentToolsResponse:
        """
        Sorted (created_at descending)

        Parameters:
            - agent_id: str.

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_agent_tools(
            agent_id="agent_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/tools"
            ),
            params=remove_none_from_dict({"limit": limit, "offset": offset}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetAgentToolsResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_agent_tool(
        self, agent_id: str, *, request: CreateToolRequest
    ) -> ResourceCreatedResponse:
        """


        Parameters:
            - agent_id: str.

            - request: CreateToolRequest.
        ---
        from julep import CreateToolRequest, CreateToolRequestType, FunctionDef
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.create_agent_tool(
            agent_id="agent_id",
            request=CreateToolRequest(
                type=CreateToolRequestType.FUNCTION,
                function=FunctionDef(
                    name="name",
                    parameters={},
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"agents/{agent_id}/tools"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceCreatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_agent_tool(
        self, agent_id: str, tool_id: str, *, function: FunctionDef
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.

            - function: FunctionDef. Function definition and parameters
        ---
        from julep import FunctionDef
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.update_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
            function=FunctionDef(
                name="name",
                parameters={},
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            json=jsonable_encoder({"function": function}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_agent_tool(self, agent_id: str, tool_id: str) -> None:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.delete_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch_agent_tool(
        self, agent_id: str, tool_id: str, *, function: PartialFunctionDef
    ) -> ResourceUpdatedResponse:
        """


        Parameters:
            - agent_id: str.

            - tool_id: str.

            - function: PartialFunctionDef. Function definition and parameters
        ---
        from julep import PartialFunctionDef
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.patch_agent_tool(
            agent_id="agent_id",
            tool_id="tool_id",
            function=PartialFunctionDef(),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"agents/{agent_id}/tools/{tool_id}",
            ),
            json=jsonable_encoder({"function": function}),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResourceUpdatedResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_job_status(self, job_id: str) -> JobStatus:
        """


        Parameters:
            - job_id: str.
        ---
        from julep.client import AsyncJulepApi

        client = AsyncJulepApi(
            api_key="YOUR_API_KEY",
        )
        await client.get_job_status(
            job_id="job_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"jobs/{job_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=300,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(JobStatus, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


def _get_base_url(
    *, base_url: typing.Optional[str] = None, environment: JulepApiEnvironment
) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception(
            "Please pass in either base_url or environment to construct the client"
        )



import json
from uuid import UUID
from typing import Optional, TypedDict
from beartype import beartype
from beartype.typing import Any, Awaitable, Dict, List, Union

from .utils import rewrap_in_class, NotSet

from ..api.types import (
    User,
    CreateDoc,
    ResourceCreatedResponse,
    ResourceUpdatedResponse,
    ListUsersResponse,
)

from .base import BaseManager
from .utils import is_valid_uuid4
from .types import DocDict


class UserCreateArgs(TypedDict):
    name: str
    about: str
    docs: List[str] = []
    metadata: Dict[str, Any] = {}


class UserUpdateArgs(TypedDict):
    name: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    about: Optional[str] = None
    overwrite: bool = False


class BaseUsersManager(BaseManager):
    """
    A manager class for handling user-related operations through an API client.

        This class provides high-level methods to interact with user records,
        such as retrieving a user by ID, creating a new user, listing all users,
        deleting a user, and updating a user's details.

        Attributes:
            api_client: The API client instance to communicate with the user service.

        Methods:
            _get: Retrieve a user by a unique UUID.
            _create: Create a new user with the specified name and about fields, and optionally additional documents.
            _list_items: List users with optional pagination through limit and offset parameters.
            _delete: Delete a user by UUID.
            _update: Update user details such as the 'about' description and name by UUID.

        Raises:
            AssertionError: If the provided ID for the user operations is not a valid UUID v4.
    """

    def _get(self, id: Union[str, UUID]) -> Union[User, Awaitable[User]]:
        """
        Get the user by their ID.

        This method is intended to retrieve a User object or an Awaitable User object by the user's unique identifier. The identifier can be a string representation of a UUID or a UUID object itself.

        Args:
            id (Union[str, UUID]): The unique identifier of the user to retrieve. It must be a valid UUID v4.

        Returns:
            Union[User, Awaitable[User]]: The retrieved User instance or an Awaitable that resolves to a User instance.

        Raises:
            AssertionError: If the `id` is not a valid UUID v4.

        Note:
            The leading underscore in the method name suggests that this method is intended for internal use and should not be a part of the public interface of the class.
        """
        assert is_valid_uuid4(id), "id must be a valid UUID v4"
        return self.api_client.get_user(user_id=id)

    def _create(
        self,
        name: str,
        about: str,
        docs: List[DocDict] = [],
        metadata: Dict[str, Any] = {},
    ) -> Union[ResourceCreatedResponse, Awaitable[ResourceCreatedResponse]]:
        # Cast docs to a list of CreateDoc objects
        """
        Create a new resource with the given name and about information, optionally including additional docs.

        This internal method allows for creating a new resource with optional docsrmation.

        Args:
            name (str): The name of the new resource.
            about (str): A brief description about the new resource.
            docs (List[DocDict], optional): A list of dictionaries with documentation-related information. Each dictionary
                must conform to the structure expected by CreateDoc. Defaults to an empty list.
            metadata (Dict[str, Any])

        Returns:
            Union[ResourceCreatedResponse, Awaitable[ResourceCreatedResponse]]: The response indicating the resource has been
            created successfully. It can be either a synchronous ResourceCreatedResponse or an asynchronous Awaitable object
            containing a ResourceCreatedResponse.

        Note:
            This method is an internal API implementation detail and should not be used directly outside the defining class
            or module.

        Side effects:
            Modifies internal state by converting each doc dict to an instance of CreateDoc and uses the
            internal API client to create a new user resource.
        """
        docs: List[CreateDoc] = [CreateDoc(**doc) for doc in docs]

        return self.api_client.create_user(
            name=name,
            about=about,
            docs=docs,
            metadata=metadata,
        )

    def _list_items(
        self,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        metadata_filter: str = "{}",
    ) -> Union[ListUsersResponse, Awaitable[ListUsersResponse]]:
        """
        Fetch a list of users, with optional pagination parameters.

            Args:
                limit (Optional[int], optional): The maximum number of users to return. Defaults to None.
                offset (Optional[int], optional): The offset from the start of the list to begin returning users. Defaults to None.

            Returns:
                Union[ListUsersResponse, Awaitable[ListUsersResponse]]: An instance of ListUsersResponse,
                or an awaitable that will resolve to it, depending on the API client implementation.
        """
        return self.api_client.list_users(
            limit=limit,
            offset=offset,
            metadata_filter=metadata_filter,
        )

    def _delete(self, user_id: Union[str, UUID]) -> Union[None, Awaitable[None]]:
        """
        Delete a user given their user ID.

            Args:
                user_id (Union[str, UUID]): The identifier of the user. Must be a valid UUID version 4.

            Returns:
                Union[None, Awaitable[None]]: None if the deletion is synchronous, or an Awaitable
                that resolves to None if the deletion is asynchronous.

            Raises:
                AssertionError: If the user_id is not a valid UUID v4.
        """
        assert is_valid_uuid4(user_id), "id must be a valid UUID v4"
        return self.api_client.delete_user(user_id=user_id)

    def _update(
        self,
        user_id: Union[str, UUID],
        about: Optional[str] = NotSet,
        name: Optional[str] = NotSet,
        metadata: Dict[str, Any] = NotSet,
        overwrite: bool = False,
    ) -> Union[ResourceUpdatedResponse, Awaitable[ResourceUpdatedResponse]]:
        """
        Update user details for a given user ID.

        This method updates the 'about' and/or 'name' fields for the user identified by user_id.

        Args:
            user_id (Union[str, UUID]): The ID of the user to be updated. Must be a valid UUID v4.
            about (Optional[str], optional): The new information about the user. Defaults to None.
            name (Optional[str], optional): The new name for the user. Defaults to None.
            metadata (Dict[str, Any])
            overwrite (bool, optional): Whether to overwrite the existing user data. Defaults to False.

        Returns:
            Union[ResourceUpdatedResponse, Awaitable[ResourceUpdatedResponse]]: The response indicating successful update or an Awaitable that resolves to such a response.

        Raises:
            AssertionError: If `user_id` is not a valid UUID v4.
        """
        assert is_valid_uuid4(user_id), "id must be a valid UUID v4"

        updateFn = (
            self.api_client.update_user if overwrite else self.api_client.patch_user
        )

        update_data = dict(
            user_id=user_id,
            about=about,
            name=name,
            metadata=metadata,
        )

        update_data = {k: v for k, v in update_data.items() if v is not NotSet}

        if not update_data:
            raise ValueError("No fields to update")

        return updateFn(**update_data)


class UsersManager(BaseUsersManager):
    """
    A class responsible for managing users in a system.

    This class is a specialized version of the BaseUsersManager and provides
    methods for retrieving, creating, listing, deleting, and updating users within
    the system.

    Methods:
        get(id: Union[str, UUID]) -> User:
            Retrieves a user based on their unique identifier (either a string or UUID).

        create(*, name: str, about: str, docs: List[DocDict]=[]) -> ResourceCreatedResponse:
            Creates a new user with the specified name, description about the user,
            and an optional list of documents associated with the user.

        list(*, limit: Optional[int]=None, offset: Optional[int]=None) -> List[User]:
            Lists users in the system, with optional limit and offset for pagination.

        delete(*, user_id: Union[str, UUID]) -> None:
            Deletes a user from the system based on their unique identifier.

        update(*, user_id: Union[str, UUID], about: Optional[str]=None, name: Optional[str]=None) -> ResourceUpdatedResponse:
            Updates an existing user's information with optional new about and name
            fields.
    """

    @beartype
    def get(self, id: Union[str, UUID]) -> User:
        """
        Retrieve a User object by its identifier.

        The method supports retrieval by both string representations of a UUID and
        UUID objects directly.

        Args:
            id (Union[str, UUID]): The identifier of the User, can be a string or UUID.

        Returns:
            User: The User object associated with the provided id.

        Raises:
            ValueError: If 'id' is neither a string nor a UUID.
            NotFoundError: If a User with the given 'id' does not exist.
        """
        return self._get(id=id)

    @beartype
    @rewrap_in_class(User)
    def create(self, **kwargs: UserCreateArgs) -> User:
        """
        Create a new resource with the specified name, about text, and associated docs.

            Args:
                name (str): The name of the resource to create.
                about (str): A brief description of the resource.
                docs (List[DocDict], optional): A list of dictionaries representing the documents associated with the resource. Defaults to an empty list.

            Returns:
                ResourceCreatedResponse: An object representing the response received upon the successful creation of the resource.

            Note:
                Using mutable types like list as default argument values in Python is risky because if the list is modified,
                those changes will persist across subsequent calls to the function which use the default value. It is
                generally safer to use `None` as a default value and then set the default inside the function if needed.

            Raises:
                BeartypeException: If the input types do not match the specified function annotations.
        """
        result = self._create(**kwargs)
        return result

    @beartype
    def list(
        self,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        metadata_filter: Dict[str, Any] = {},
    ) -> List[User]:
        """
        Lists the users optionally applying limit and offset.

        Args:
            limit (Optional[int], optional): The maximum number of users to return.
                None means no limit. Defaults to None.
            offset (Optional[int], optional): The index of the first user to return.
                None means start from the beginning. Defaults to None.

        Returns:
            List[User]: A list of user objects.

        Raises:
            BeartypeException: If the type of `limit` or `offset` is not as expected.
        """
        metadata_filter_string = json.dumps(metadata_filter)

        return self._list_items(
            limit=limit,
            offset=offset,
            metadata_filter=metadata_filter_string,
        ).items

    @beartype
    def delete(
        self,
        user_id: Union[str, UUID],
    ) -> None:
        """
        Deletes a user based on the provided user ID.

        Args:
            user_id (Union[str, UUID]): Unique identifier of the user.

        Returns:
            None

        Note:
            This function is type-checked with `beartype` to ensure that the `user_id`
            parameter matches either a string or a UUID type.

        Raises:
            The specific exceptions raised depend on the implementation of the `_delete`
            method this function proxies to.
        """
        return self._delete(
            user_id=user_id,
        )

    @beartype
    @rewrap_in_class(User)
    def update(self, *, user_id: Union[str, UUID], **kwargs: UserUpdateArgs) -> User:
        """
        Update user information.

        This method updates user details such as the `about` text and user's `name` for a given `user_id`.

        Args:
            user_id (Union[str, UUID]): The unique identifier for the user, which can be a string or a UUID object.
            about(Optional[str], optional): The descriptive information about the user. Defaults to None, indicating that `about` should not be updated if not provided.
            name(Optional[str], optional): The name of the user. Defaults to None, indicating that `name` should not be updated if not provided.
            overwrite(bool, optional): Whether to overwrite the existing user data. Defaults to False.

        Returns:
            ResourceUpdatedResponse: An object indicating the outcome of the update operation, which typically includes the status of the operation and possibly the updated resource data.
        """
        result = self._update(user_id=user_id, **kwargs)
        return result


class AsyncUsersManager(BaseUsersManager):
    """
    A class that provides asynchronous management of users extending BaseUsersManager.

    Attributes are inherited from BaseUsersManager.

    Methods:
        get (Union[UUID, str]) -> User:
            Asynchronously retrieves a user by their unique identifier (either a UUID or a string).

        create (*, name: str, about: str, docs: List[DocDict]=[]) -> ResourceCreatedResponse:
            Asynchronously creates a new user with the given name, about description, and optional list of documents.

        list (*, limit: Optional[int]=None, offset: Optional[int]=None) -> List[User]:
            Asynchronously retrieves a list of users with an optional limit and offset for pagination.

        delete (*, user_id: Union[str, UUID]) -> None:
            Asynchronously deletes a user identified by their unique ID (either a string or a UUID).

        update (*, user_id: Union[str, UUID], about: Optional[str]=None, name: Optional[str]=None) -> ResourceUpdatedResponse:
            Asynchronously updates a user's information identified by their unique ID with optional new about description and name.

    Note:
        The beartype decorator is used for runtime type checking of the parameters passed to the methods.
    """

    @beartype
    async def get(self, id: Union[UUID, str]) -> User:
        """
        Fetch a User object asynchronously by its identifier.

        This method retrieves a User object from some data storage asynchronously based on the provided identifier, which can be either a UUID or a string.

        Args:
            id (Union[UUID, str]): The unique identifier of the User to be retrieved.

        Returns:
            User: An instance of the User class corresponding to the given id.

        Raises:
            Exception: If the retrieval fails or the user cannot be found.
        """
        return await self._get(id=id)

    @beartype
    @rewrap_in_class(User)
    async def create(self, **kwargs: UserCreateArgs) -> User:
        """
        Asynchronously create a new resource with the provided name, description, and documents.

        This function is decorated with `@beartype` to ensure type checking at runtime.

        Args:
            name (str): The name of the resource to be created.
            about (str): Brief information about the resource.
            docs (List[DocDict], optional): A list of document dictionaries with structure defined by DocDict type.

        Returns:
            ResourceCreatedResponse: An instance representing the response after resource creation.

        Raises:
            BeartypeException: If any of the parameters do not match their annotated types.
        """
        result = await self._create(**kwargs)
        return result

    @beartype
    async def list(
        self,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        metadata_filter: Dict[str, Any] = {},
    ) -> List[User]:
        """
        Asynchronously lists users with optional limits and offsets.

        This function applies the `beartype` decorator for runtime type checking.

        Args:
            limit (Optional[int], optional): The maximum number of users to be returned. Defaults to None, which means no limit.
            offset (Optional[int], optional): The number to offset the list of returned users by. Defaults to None, which means no offset.

        Returns:
            List[User]: A list of user objects.

        Raises:
            TypeError: If any input arguments are not of the expected type.
            Any other exception that might be raised during the retrieval of users from the data source.

        Note:
            The actual exception raised by the `beartype` decorator or during the users' retrieval will depend on the implementation detail of the `self._list_items` method and the `beartype` configuration.
        """
        metadata_filter_string = json.dumps(metadata_filter)

        return (
            await self._list_items(
                limit,
                offset,
                metadata_filter=metadata_filter_string,
            )
        ).items

    @beartype
    async def delete(
        self,
        user_id: Union[str, UUID],
    ) -> None:
        """
        Asynchronously deletes a user by their user ID.

        Args:
            user_id (Union[str, UUID]): The unique identifier of the user to delete, which can be a string or a UUID.

        Returns:
            None: This function does not return anything.

        Notes:
            - The function is decorated with `@beartype` for runtime type checking.
            - This function is a coroutine, it should be called with `await`.

        Raises:
            - The raised exceptions depend on the implementation of the underlying `_delete` coroutine.
        """
        return await self._delete(
            user_id=user_id,
        )

    @beartype
    @rewrap_in_class(User)
    async def update(
        self, *, user_id: Union[str, UUID], **kwargs: UserUpdateArgs
    ) -> User:
        """
        Asynchronously updates user details.

        This function updates user details such as 'about' and 'name'. It uses type annotations to enforce the types of the parameters and an asynchronous call to '_update' method to perform the actual update operation.

        Args:
            user_id (Union[str, UUID]): The unique identifier of the user, which can be either a string or a UUID.
            about (Optional[str], optional): A description of the user. Default is None, indicating no update.
            name (Optional[str], optional): The name of the user. Default is None, indicating no update.

        Returns:
            ResourceUpdatedResponse: An object representing the update status.

        Note:
            This function is decorated with @beartype to perform runtime type checking.
        """
        result = await self._update(user_id=user_id, **kwargs)
        return result



from uuid import UUID

from typing import Optional
from beartype import beartype
from beartype.typing import Awaitable, List, Union

from ..api.types import (
    Memory,
    GetAgentMemoriesResponse,
)

from .base import BaseManager
from .utils import is_valid_uuid4


class BaseMemoriesManager(BaseManager):
    """
    A base manager class for handling agent memories.

        This manager provides an interface to interact with agent memories, facilitating
        operations such as listing and retrieving memories based on various criteria.

        Methods:
            _list(agent_id, query, types=None, user_id=None, limit=None, offset=None):
                Retrieves a list of memories for a given agent.

        Args:
            agent_id (str): A valid UUID v4 string identifying the agent.
            query (str): The query string to search memories.
            types (Optional[Union[str, List[str]]]): The type(s) of memories to retrieve.
            user_id (Optional[str]): The user identifier associated with the memories.
            limit (Optional[int]): The maximum number of memories to retrieve.
            offset (Optional[int]): The number of initial memories to skip in the result set.

        Returns:
            Union[GetAgentMemoriesResponse, Awaitable[GetAgentMemoriesResponse]]:
                A synchronous or asynchronous response object containing the list of agent memories.

        Raises:
            AssertionError: If `agent_id` is not a valid UUID v4.
    """

    def _list(
        self,
        agent_id: str,
        query: str,
        types: Optional[Union[str, List[str]]] = None,
        user_id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> Union[GetAgentMemoriesResponse, Awaitable[GetAgentMemoriesResponse]]:
        """
        List memories from a given agent based on a query and further filtering options.

        Args:
            agent_id (str): A valid UUID v4 representing the agent ID.
            query (str): Query string to filter memories.
            types (Optional[Union[str, List[str]]], optional): The types of memories to filter.
            user_id (Optional[str], optional): The user ID to filter memories.
            limit (Optional[int], optional): The maximum number of memories to return.
            offset (Optional[int], optional): The number of memories to skip before starting to collect the result set.

        Returns:
            Union[GetAgentMemoriesResponse, Awaitable[GetAgentMemoriesResponse]]: Returns a synchronous or asynchronous response with the agent memories.

        Raises:
            AssertionError: If `agent_id` is not a valid UUID v4.
        """
        assert is_valid_uuid4(agent_id), "agent_id must be a valid UUID v4"

        return self.api_client.get_agent_memories(
            agent_id=agent_id,
            query=query,
            user_id=user_id,
            limit=limit,
            offset=offset,
        )


class MemoriesManager(BaseMemoriesManager):
    """
    A class for managing memory entities associated with agents.

        Inherits from `BaseMemoriesManager` and extends its functionality to specifically
        manage and retrieve memory entities for agents based on query parameters.

        Attributes:
            Inherited from `BaseMemoriesManager`.

        Methods:
            list: Retrieves a list of memory entities based on query parameters.
    """

    @beartype
    def list(
        self,
        *,
        agent_id: Union[str, UUID],
        query: str,
        types: Optional[Union[str, List[str]]] = None,
        user_id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> List[Memory]:
        """
        List memories meeting specified criteria.

        This function fetches a list of Memory objects based on various filters and parameters such as agent_id, query, types, user_id, limit, and offset.

        Args:
            agent_id (Union[str, UUID]): The unique identifier for the agent.
            query (str): The search term used to filter memories.
            types (Optional[Union[str, List[str]]], optional): The types of memories to retrieve. Can be a single type as a string or a list of types. Default is None, which does not filter by type.
            user_id (Optional[str], optional): The unique identifier for the user. If provided, only memories associated with this user will be retrieved. Default is None.
            limit (Optional[int], optional): The maximum number of memories to return. Default is None, which means no limit.
            offset (Optional[int], optional): The number of memories to skip before starting to return the results. Default is None.

        Returns:
            List[Memory]: A list of Memory objects that match the given criteria.

        Note:
            The `@beartype` decorator is used to ensure that arguments conform to the expected types at runtime.
        """
        return self._list(
            agent_id=agent_id,
            query=query,
            types=types,
            user_id=user_id,
            limit=limit,
            offset=offset,
        ).items


class AsyncMemoriesManager(BaseMemoriesManager):
    """
    Asynchronously lists memories based on various filter parameters.

            Args:
                agent_id (Union[str, UUID]): The unique identifier of the agent.
                query (str): The search query string to filter memories.
                types (Optional[Union[str, List[str]]], optional): The types of memories to filter by. Defaults to None.
                user_id (Optional[str], optional): The unique identifier of the user. Defaults to None.
                limit (Optional[int], optional): The maximum number of memories to return. Defaults to None.
                offset (Optional[int], optional): The number of memories to skip before starting to collect the result set. Defaults to None.

            Returns:
                List[Memory]: A list of Memory objects that match the given filters.

            Raises:
                ValidationError: If the input validation fails.
                DatabaseError: If there is a problem accessing the database.
    """

    @beartype
    async def list(
        self,
        *,
        agent_id: Union[str, UUID],
        query: str,
        types: Optional[Union[str, List[str]]] = None,
        user_id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> List[Memory]:
        """
        Asynchronously list memories based on query parameters.

        Args:
            agent_id (Union[str, UUID]): The ID of the agent to list memories for.
            query (str): The query string to filter memories.
            types (Optional[Union[str, List[str]]], optional): The types of memories to retrieve. Defaults to None.
            user_id (Optional[str], optional): The ID of the user to list memories for. Defaults to None.
            limit (Optional[int], optional): The maximum number of memories to return. Defaults to None.
            offset (Optional[int], optional): The offset to start listing memories from. Defaults to None.

        Returns:
            List[Memory]: A list of Memory objects that match the query.

        Note:
            `@beartype` decorator is used for runtime type checking.
        """
        return (
            await self._list(
                agent_id=agent_id,
                query=query,
                types=types,
                user_id=user_id,
                limit=limit,
                offset=offset,
            )
        ).items



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class JobStatusState(str, enum.Enum):
    """
    Current state (one of: pending, in_progress, retrying, succeeded, aborted, failed)
    """

    PENDING = "pending"
    IN_PROGRESS = "in_progress"
    RETRYING = "retrying"
    SUCCEEDED = "succeeded"
    ABORTED = "aborted"
    FAILED = "failed"
    UNKNOWN = "unknown"

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        in_progress: typing.Callable[[], T_Result],
        retrying: typing.Callable[[], T_Result],
        succeeded: typing.Callable[[], T_Result],
        aborted: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is JobStatusState.PENDING:
            return pending()
        if self is JobStatusState.IN_PROGRESS:
            return in_progress()
        if self is JobStatusState.RETRYING:
            return retrying()
        if self is JobStatusState.SUCCEEDED:
            return succeeded()
        if self is JobStatusState.ABORTED:
            return aborted()
        if self is JobStatusState.FAILED:
            return failed()
        if self is JobStatusState.UNKNOWN:
            return unknown()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .chat_settings_response_format_schema import ChatSettingsResponseFormatSchema
from .chat_settings_response_format_type import ChatSettingsResponseFormatType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ChatSettingsResponseFormat(pydantic.BaseModel):
    """
    An object specifying the format that the model must output.

    Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.

    **Important:** when using JSON mode, you **must** also instruct the model to produce JSON yourself via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request. Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    """

    type: typing.Optional[ChatSettingsResponseFormatType] = pydantic.Field(
        description='Must be one of `"text"`, `"regex"` or `"json_object"`.'
    )
    pattern: typing.Optional[str] = pydantic.Field(
        description='Regular expression pattern to use if `type` is `"regex"`'
    )
    schema_: typing.Optional[ChatSettingsResponseFormatSchema] = pydantic.Field(
        alias="schema", description='JSON Schema to use if `type` is `"json_object"`'
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ChatMlMessageRole(str, enum.Enum):
    """
    ChatML role (system|assistant|user|function_call|function)
    """

    USER = "user"
    ASSISTANT = "assistant"
    SYSTEM = "system"
    FUNCTION_CALL = "function_call"
    FUNCTION = "function"

    def visit(
        self,
        user: typing.Callable[[], T_Result],
        assistant: typing.Callable[[], T_Result],
        system: typing.Callable[[], T_Result],
        function_call: typing.Callable[[], T_Result],
        function: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ChatMlMessageRole.USER:
            return user()
        if self is ChatMlMessageRole.ASSISTANT:
            return assistant()
        if self is ChatMlMessageRole.SYSTEM:
            return system()
        if self is ChatMlMessageRole.FUNCTION_CALL:
            return function_call()
        if self is ChatMlMessageRole.FUNCTION:
            return function()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .user_metadata import UserMetadata

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class User(pydantic.BaseModel):
    name: typing.Optional[str] = pydantic.Field(description="Name of the user")
    about: typing.Optional[str] = pydantic.Field(description="About the user")
    created_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="User created at (RFC-3339 format)"
    )
    updated_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="User updated at (RFC-3339 format)"
    )
    id: str = pydantic.Field(description="User id (UUID)")
    metadata: typing.Optional[UserMetadata] = pydantic.Field(
        description="(Optional) metadata"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
